@@ -3,6 +3,7 @@ use 5.010;
 use strict;
 use warnings;
 use boolean ();
+use Carp (qw/croak shortmess/);
 use Exporter::Tidy all => [
     qw( encode_bifcode
       decode_bifcode
@@ -12,16 +13,14 @@ use Exporter::Tidy all => [
 
 # ABSTRACT: Serialisation similar to Bencode + undef/UTF8
 
-our $VERSION = '2.000_10';
+our $VERSION = '2.000_1';
 our $max_depth;
 our @CARP_NOT = (__PACKAGE__);
 
 sub _croak {
-    require Carp;
-    my $type     = shift // Carp::croak('usage: _croak($TYPE, [$msg])');
+    my $type     = shift // croak 'usage: _error($TYPE, [$msg])';
     my %messages = (
         Decode             => 'garbage at',
-        DecodeBifcodeTerm  => 'missing BIFCODE terminator at',
         DecodeBytes        => 'malformed BYTES length at',
         DecodeBytesTrunc   => 'unexpected BYTES end of data at',
         DecodeBytesTerm    => 'missing BYTES termination at',
@@ -37,87 +36,53 @@ sub _croak {
         DecodeUTF8Term     => 'missing UTF8 termination at',
         DecodeUsage        => undef,
         DiffUsage          => 'usage: diff_bifcode($b1, $b2, [$diff_args])',
-        EncodeBytesUndef   => 'Bifcode::BYTES ref is undefined',
+        EncodeBytesUndef   => 'Bifcode::V2::BYTES ref is undefined',
         EncodeReal         => undef,
-        EncodeRealUndef    => 'Bifcode::REAL ref is undefined',
+        EncodeRealUndef    => 'Bifcode::V2::REAL ref is undefined',
         EncodeInteger      => undef,
-        EncodeIntegerUndef => 'Bifcode::INTEGER ref is undefined',
+        EncodeIntegerUndef => 'Bifcode::V2::INTEGER ref is undefined',
         DecodeKeyType      => 'dict key is not BYTES or UTF8 at',
         DecodeKeyDuplicate => 'duplicate dict key at',
         DecodeKeyOrder     => 'dict key not in sort order at',
         DecodeKeyValue     => 'dict key is missing value at',
-        EncodeUTF8Undef    => 'Bifcode::UTF8 ref is undefined',
+        EncodeUTF8Undef    => 'Bifcode::V2::UTF8 ref is undefined',
         EncodeUnhandled    => undef,
         EncodeUsage        => 'usage: encode_bifcode($arg)',
         ForceUsage         => 'ref and type must be defined',
     );
 
-    my $err = 'Bifcode::Error::' . $type;
-    my $msg = shift // $messages{$type}
-      // Carp::croak("Bifcode::_croak($type) has no message ");
-    my $short = Carp::shortmess('');
+    my $err   = 'Bifcode::V2::Error::' . $type;
+    my $msg   = shift // $messages{$type} // '(no message)';
+    my $short = shortmess('');
 
     $msg =~ s! at$!' at input byte '. ( pos() // 0 )!e;
 
-    eval qq[
-        package $err;
+    eval 'package ' . $err . qq[;
         use overload
           bool => sub { 1 },
           '""' => sub { \${ \$_[0] } . ' (' . ( ref \$_[0] ) . ')$short' },
           fallback => 1;
-        1;
-        ];
+        1; ];
 
-    die $@ if $@;
     die bless \$msg, $err;
 }
 
-my $chunk = qr/ \G (?|
+my $match = qr/ \G (?|
       (~,)
     | (f,)
     | (t,)
-    | (B|b|u) (?:     ( 0 |    [1-9]   [0-9]* ) \. )?
-    | (i)     (?:     ( 0 | -? [1-9]   [0-9]* ) ,  )?
-    | (r)     (?:     ( 0 | -? [1-9]   [0-9]* )
-                \.    ( 0 |    [0-9]*  [1-9]  )
-                e ( (?: 0 | -? [1-9] ) [0-9]* ) , 
-                                                   )?
+    | (b|u) (?: ( 0 | [1-9] [0-9]* ) \. )?
+    | (i)   (?: ( 0 | -? [1-9] [0-9]* ) , )?
+    | (r)   (?: ( 0 | -? [1-9] [0-9]* ) \. ( 0 | [0-9]* [1-9] ) e
+                ( (?: 0 | -? [1-9] ) [0-9]* ) , )?
     | (\[)
     | (\{)
 ) /x;
 
-sub _decode_bifcode_key {
-
-    unless (m/ \G (b|u) (?: ( 0 | [1-9] [0-9]* ) \. )? /gcx) {
-        _croak m/ \G \z /xgc ? 'DecodeTrunc' : 'DecodeKeyType';
-    }
-
-    if ( $1 eq 'b' ) {
-        my $len = $2 // _croak 'DecodeBytes';
-        _croak 'DecodeBytesTrunc' if $len > length() - pos();
-
-        my $data = substr $_, pos(), $len;
-        pos() = pos() + $len;
-
-        _croak 'DecodeBytesTerm' unless m/ \G : /xgc;
-        return $data;
-    }
-    elsif ( $1 eq 'u' ) {
-        my $len = $2 // _croak 'DecodeUTF8';
-        _croak 'DecodeUTF8Trunc' if $len > length() - pos();
-
-        utf8::decode( my $str = substr $_, pos(), $len );
-        pos() = pos() + $len;
-
-        _croak 'DecodeUTF8Term' unless m/ \G : /xgc;
-        return $str;
-    }
-}
-
 sub _decode_bifcode_chunk {
     local $max_depth = $max_depth - 1 if defined $max_depth;
 
-    unless (m/$chunk/gc) {
+    unless (m/$match/gc) {
         _croak m/ \G \z /xgc ? 'DecodeTrunc' : 'Decode';
     }
 
@@ -151,7 +116,7 @@ sub _decode_bifcode_chunk {
         return $str;
     }
     elsif ( $1 eq 'i' ) {
-        return 0 + $2               if defined $2;
+        return $2 if defined $2;
         _croak 'DecodeIntegerTrunc' if m/ \G \z /xgc;
         _croak 'DecodeInteger';
     }
@@ -165,7 +130,7 @@ sub _decode_bifcode_chunk {
           and $3 eq '0'     # mantissa 0.0
           and $4 ne '0';    # sign or exponent 0.0e0
 
-        return 0.0 + ( $2 . '.' . $3 . 'e' . $4 );
+        return $2 . '.' . $3 . 'e' . $4;
     }
     elsif ( $1 eq '[' ) {
         _croak 'DecodeDepth' if defined $max_depth and $max_depth < 0;
@@ -183,8 +148,10 @@ sub _decode_bifcode_chunk {
         my %hash;
         until (m/ \G \} /xgc) {
             _croak 'DecodeTrunc' if m/ \G \z /xgc;
+            _croak 'DecodeKeyType' unless m/ \G (b|u) /xgc;
 
-            my $key = _decode_bifcode_key();
+            pos() = pos() - 1;
+            my $key = _decode_bifcode_chunk();
 
             _croak 'DecodeKeyDuplicate' if exists $hash{$key};
             _croak 'DecodeKeyOrder'
@@ -196,15 +163,6 @@ sub _decode_bifcode_chunk {
         }
         return \%hash;
     }
-    elsif ( $1 eq 'B' ) {
-        my $len = $2 // _croak 'DecodeBifcode';
-        _croak 'DecodeBifcodeTrunc' if $len > length() - pos();
-
-        my $res = _decode_bifcode_chunk();
-        _croak 'DecodeBifcodeTerm' unless m/ \G , /xgc;
-
-        return $res;
-    }
 }
 
 sub decode_bifcode {
@@ -218,7 +176,7 @@ sub decode_bifcode {
       if utf8::is_utf8($_);
 
     my $deserialised_data = _decode_bifcode_chunk();
-    _croak 'DecodeTrailing', " For: $_" if $_ !~ m/ \G \z /xgc;
+    _croak 'DecodeTrailing' if $_ !~ m/ \G \z /xgc;
     return $deserialised_data;
 }
 
@@ -229,7 +187,7 @@ my $number_qr = qr/\A ( 0 | -? [1-9] [0-9]* )
 sub _encode_bifcode {
     map {
         if ( !defined $_ ) {
-            '~' . ',';
+            '~,';
         }
         elsif ( ( my $ref = ref $_ ) eq '' ) {
             if ( $_ =~ $number_qr ) {
@@ -245,16 +203,10 @@ sub _encode_bifcode {
                     'i' . $_ . ',';
                 }
             }
-            elsif ( utf8::is_utf8($_) ) {
+            else {
                 utf8::encode( my $str = $_ );
                 'u' . length($str) . '.' . $str . ',';
             }
-            elsif ( $_ =~ m/^[\x{20}-\x{7E}]*$/ ) {
-                'u' . length($_) . '.' . $_ . ',';
-            }
-            else {
-                'b' . length($_) . '.' . $_ . ',';
-            }
         }
         elsif ( $ref eq 'ARRAY' ) {
             '[' . join( '', map _encode_bifcode($_), @$_ ) . ']';
@@ -263,40 +215,36 @@ sub _encode_bifcode {
             '{' . join(
                 '',
                 do {
-                    my $k;
                     my @k = sort keys %$_;
-
                     map {
-                        $k = shift @k;
+                        my $k = shift @k;
 
-                        if ( $k =~ m/^[\x{20}-\x{7E}]*$/ ) {
-                            ( 'u' . length($k) . '.' . $k . ':', $_ );
-                        }
-                        elsif ( utf8::is_utf8($k) ) {
-                            utf8::encode($k);
-                            ( 'u' . length($k) . '.' . $k . ':', $_ );
-                        }
-                        else {
-                            ( 'b' . length($k) . '.' . $k . ':', $_ );
-                        }
+                        # if ( is valid utf8($k) ) {
+                        utf8::encode($k);
+                        ( 'u' . length($k) . '.' . $k . ',', $_ );
+
+                        # }
+                        # else {
+                        #     ('b' . length($k) . '.' . $k .',', $_);
+                        # }
                     } _encode_bifcode( @$_{@k} );
-                }
+                  }
             ) . '}';
         }
-        elsif ( $ref eq 'SCALAR' or $ref eq 'Bifcode::BYTES' ) {
+        elsif ( $ref eq 'SCALAR' or $ref eq 'Bifcode::V2::BYTES' ) {
             $$_ // _croak 'EncodeBytesUndef';
             'b' . length($$_) . '.' . $$_ . ',';
         }
         elsif ( boolean::isBoolean($_) ) {
-            ( $_ ? 't' : 'f' ) . ',';
+            $_ ? 't,' : 'f,';
         }
-        elsif ( $ref eq 'Bifcode::INTEGER' ) {
+        elsif ( $ref eq 'Bifcode::V2::INTEGER' ) {
             $$_ // _croak 'EncodeIntegerUndef';
             _croak 'EncodeInteger', 'invalid integer: ' . $$_
               unless $$_ =~ m/\A (?: 0 | -? [1-9] [0-9]* ) \z/x;
             sprintf 'i%s,', $$_;
         }
-        elsif ( $ref eq 'Bifcode::REAL' ) {
+        elsif ( $ref eq 'Bifcode::V2::REAL' ) {
             $$_ // _croak 'EncodeRealUndef';
             _croak 'EncodeReal', 'invalid real: ' . $$_
               unless $$_ =~ $number_qr;
@@ -306,9 +254,9 @@ sub _encode_bifcode {
             $x =~ s/ ([1-9]) (0+ e)/.${1}e/x;    # remove trailing zeros
             $x;
         }
-        elsif ( $ref eq 'Bifcode::UTF8' ) {
+        elsif ( $ref eq 'Bifcode::V2::UTF8' ) {
             my $str = $$_ // _croak 'EncodeUTF8Undef';
-            utf8::encode($str);
+            utf8::encode($str);  #, sub { croak 'invalid Bifcode::V2::UTF8' } );
             'u' . length($str) . '.' . $str . ',';
         }
         else {
@@ -318,16 +266,8 @@ sub _encode_bifcode {
 }
 
 sub encode_bifcode {
-    if ( ( @_ == 2 ) && pop ) {
-        my $b = (&_encode_bifcode)[0];
-        'B' . length($b) . '.' . $b . ',';
-    }
-    elsif ( @_ == 1 ) {
-        (&_encode_bifcode)[0];
-    }
-    else {
-        _croak 'EncodeUsage';
-    }
+    _croak 'EncodeUsage' if @_ != 1;
+    (&_encode_bifcode)[0];
 }
 
 sub force_bifcode {
@@ -335,7 +275,7 @@ sub force_bifcode {
     my $type = shift;
 
     _croak 'ForceUsage' unless defined $ref and defined $type;
-    bless \$ref, 'Bifcode::' . uc($type);
+    bless \$ref, 'Bifcode::V2::' . uc($type);
 }
 
 sub _expand_bifcode {
@@ -343,7 +283,7 @@ sub _expand_bifcode {
     $bifcode =~ s/ (
             [\[\]\{\}]
             | ~,
-            | (B|u|b) [0-9]+ \.
+            | (u|b) [0-9]+ \.
             | r -? [0-9]+ \. [0-9]+ e -? [0-9]+ ,
             | i [0-9]+ ,
         ) /\n$1/gmx;
@@ -364,61 +304,6 @@ sub diff_bifcode {
     return Text::Diff::diff( \$b1, \$b2, $diff_args );
 }
 
-# Looking for something like "B48." that may be truncated at any point
-my $qr_bcbc = qr/
-    ^
-    (                                   # 1
-        B
-        (?:
-            \Z | (?:
-                ( 0 | [1-9][0-9]* )     # 2
-                ( \Z | (\.) )           # 3
-            )
-        )
-    )
-/x;
-
-sub anyevent_read_type {
-    my ( $self, $cb, $maxdepth ) = @_;
-
-    sub {
-        return unless defined $_[0]->{rbuf};
-
-        $_[0]->{rbuf} =~ s/^[\r\n]*//;
-        return unless length $_[0]->{rbuf};
-
-        $_[0]->{rbuf} =~ $qr_bcbc;
-
-        if ( length $3 ) {
-            $_[0]->unshift_read(
-                chunk => length($1) + $2 + 1,
-                sub {
-                    my $data = eval { decode_bifcode( $_[1], $maxdepth ) };
-                    if ($@) {
-                        $_[0]->_error( Errno::EBADMSG(), undef, $@ );
-                    }
-                    else {
-                        $cb->( $_[0], $data );
-                    }
-                    1;
-                }
-            );
-            return 1;
-        }
-        elsif ( not length $1 ) {
-            $_[0]->_error( Errno::EBADMSG() );
-            return;
-        }
-        else {
-            return;    # not enough data yet
-        }
-    };
-}
-
-sub anyevent_write_type {
-    encode_bifcode( $_[1], 1 ) . "\n";
-}
-
 1;
 
 __END__
@@ -429,11 +314,11 @@ __END__
 
 =head1 NAME
 
-Bifcode - encode and decode BIFCODE serialization format
+Bifcode::V2 - simple serialization format (version 2)
 
 =head1 VERSION
 
-2.000_10 (2020-01-31)
+2.000_1 (2019-09-02)
 
 =head1 SYNOPSIS
 
@@ -445,25 +330,28 @@ Bifcode - encode and decode BIFCODE serialization format
         bools   => [ boolean::false, boolean::true, ],
         bytes   => \pack( 's<',       255 ),
         integer => 25,
-        real    => 1.25e-5,
-        null    => undef,
+        float   => 1.25e-5,
+        undef   => undef,
         utf8    => "Ελύτη",
     };
 
-    # 7b 75 35 2e 62 6f 6f 6c 73 3a 5b 66 2c 74 2c    {u5.bools:[f,t,
-    # 5d 75 35 2e 62 79 74 65 73 3a 62 32 2e ff  0    ]u5.bytes:b2...
-    # 2c 75 37 2e 69 6e 74 65 67 65 72 3a 69 32 35    ,u7.integer:i25
-    # 2c 75 34 2e 6e 75 6c 6c 3a 7e 2c 75 34 2e 72    ,u4.null:~,u4.r
-    # 65 61 6c 3a 72 31 2e 32 35 65 2d 35 2c 75 34    eal:r1.25e-5,u4
-    # 2e 75 74 66 38 3a 75 31 30 2e ce 95 ce bb cf    .utf8:u10......
-    # 8d cf 84 ce b7 2c 7d                            .....,}
+    # 7b 75 35 2e 62 6f 6f 6c 73 2c 5b 66    {u5.bools,[f
+    # 2c 74 2c 5d 75 35 2e 62 79 74 65 73    ,t,]u5.bytes
+    # 2c 62 32 2e ff  0 2c 75 35 2e 66 6c    ,b2...,u5.fl
+    # 6f 61 74 2c 72 31 2e 32 35 65 2d 35    oat,r1.25e-5
+    # 2c 75 37 2e 69 6e 74 65 67 65 72 2c    ,u7.integer,
+    # 69 32 35 2c 75 35 2e 75 6e 64 65 66    i25,u5.undef
+    # 2c 7e 2c 75 34 2e 75 74 66 38 2c 75    ,~,u4.utf8,u
+    # 31 30 2e ce 95 ce bb cf 8d cf 84 ce    10..........
+    # b7 2c 7d                               .,}
 
     my $decoded = decode_bifcode $bifcode;
 
 =head1 DESCRIPTION
 
-B<Bifcode::V2> implements the I<BIFCODE> (v2) serialisation format, a
-mixed binary/text encoding with support for the following data types:
+B<Bifcode::V2> implements the I<bifcode version 2> serialisation
+format, a mixed binary/text encoding with support for the following
+data types:
 
 =over
 
@@ -502,7 +390,7 @@ is no need to escape special characters in strings. It is not
 considered human readable, but as it is mostly text it can usually be
 visually debugged.
 
-I<BIFCODE> can only be constructed canonically; i.e. there is only one
+I<bifcode> can only be constructed canonically; i.e. there is only one
 possible encoding per data structure. This property makes it suitable
 for comparing structures (using cryptographic hashes) across networks.
 
@@ -512,7 +400,7 @@ with the same features.
 
 =head1 MOTIVATION
 
-I<BIFCODE> was created for a project because none of currently
+I<bifcode> was created for a project because none of currently
 available serialization formats (Bencode, JSON, MsgPack, Netstrings,
 Sereal, YAML, etc) met the requirements of:
 
@@ -578,40 +466,28 @@ extraneous trailing zero, such as 'r3.10e0,', are invalid.
 
 =head2 BIFCODE_LIST
 
-Lists are encoded as a '[' followed by their elements (also I<BIFCODE>
+Lists are encoded as a '[' followed by their elements (also I<bifcode>
 encoded) followed by a ']'. For example '[u4.spam,u4.eggs,]'
 corresponds to ['spam', 'eggs'].
 
 =head2 BIFCODE_DICT
 
 Dictionaries are encoded as a '{' followed by a list of alternating
-keys and their corresponding values followed by a '}'. Keys must be of
-type BIFCODE_UTF8 or BIFCODE_BYTES and are encoded with a ":" as the
-last character instead of ",".
-
-For example, '{u3.cow:u3.moo,u4.spam:u4.eggs,}' corresponds to {'cow':
-'moo', 'spam': 'eggs'} and '{u4.spam:[u1.a,u1.b,]}' corresponds to
-{'spam'.  ['a', 'b']}. Keys must appear in sorted order (sorted as raw
-strings, not alphanumerics).
-
-=head2 BIFCODE_BIFCODE
-
-A Bifcode string is "B" followed by the octet length of the encoded
-string as a base ten number followed by a "." and the encoded string
-followed by ",". This is typically used to frame Bifcode structures
-over a network.
+keys and their corresponding values followed by a '}'. For example,
+'{u3.cow,u3.moo,u4.spam,u4.eggs,}' corresponds to {'cow': 'moo',
+'spam': 'eggs'} and '{u4.spam,[u1.a,u1.b,]}' corresponds to {'spam'.
+['a', 'b']}. Keys must be BIFCODE_UTF8 or BIFCODE_BYTES and appear in
+sorted order (sorted as raw strings, not alphanumerics).
 
 =head1 INTERFACE
 
-=head2 C<encode_bifcode( $datastructure [, $enclose ] )>
+=head2 C<encode_bifcode( $datastructure )>
 
-The first argument (required) may be a scalar, or may be a reference to
-either a scalar, an array, or a hash. Arrays and hashes may in turn
-contain values of these same types. Returns the appropriate BIFCODE_*
-byte string. If $enclose is true then the result is further encoded as
-BIFCODE_BIFCODE.
+Takes a single argument which may be a scalar, or may be a reference to
+either a scalar, an array or a hash. Arrays and hashes may in turn
+contain values of these same types. Returns a byte string.
 
-The mapping from Perl to I<BIFCODE> is as follows:
+The mapping from Perl to I<bifcode> is as follows:
 
 =over
 
@@ -620,37 +496,21 @@ The mapping from Perl to I<BIFCODE> is as follows:
 =item * The C<true> and C<false> values from the L<boolean>
 distribution encode to BIFCODE_TRUE and BIFCODE_FALSE.
 
-=item * A plain scalar is treated as follows:
+=item * Plain scalars are treated as BIFCODE_UTF8 unless:
 
 =over
 
 =item
 
-BIFCODE_UTF8 if C<utf8::is_utf8> returns true; or
+They look like canonically represented integers in which case they are
+mapped to BIFCODE_INTEGER; or
 
 =item
 
-BIFCODE_INTEGER if it looks like a canonically represented integer; or
-
-=item
-
-BIFCODE_REAL if it looks like a real number; or
-
-=item
-
-BIFCODE_UTF8 if it only contains ASCII characters; or
-
-=item
-
-BIFCODE_BYTES when none of the above applies.
+They look like reals in which case they are mapped to BIFCODE_REAL.
 
 =back
 
-You can force scalars to be encoded a particular way by passing a
-reference to them blessed as Bifcode::BYTES, Bifcode::INTEGER,
-Bifcode::REAL or Bifcode::UTF8. The C<force_bifcode> function below can
-help with creating such references.
-
 =item * SCALAR references become BIFCODE_BYTES.
 
 =item * ARRAY references become BIFCODE_LIST.
@@ -659,6 +519,11 @@ help with creating such references.
 
 =back
 
+You can force scalars to be encoded a particular way by passing a
+reference to them blessed as Bifcode::V2::BYTES, Bifcode::V2::INTEGER,
+Bifcode::V2::REAL or Bifcode::V2::UTF8. The C<force_bifcode> function
+below can help with creating such references.
+
 This subroutine croaks on unhandled data types.
 
 =head2 C<decode_bifcode( $string [, $max_depth ] )>
@@ -670,183 +535,159 @@ If you pass an integer for the second option, it will croak when
 attempting to parse dictionaries nested deeper than this level, to
 prevent DoS attacks using maliciously crafted input.
 
-I<BIFCODE> types are mapped back to Perl in the reverse way to the
-C<encode_bifcode> function, except for:
-
-=over
-
-=item * Any scalars which were "forced"
-to a particular type (using blessed references) will decode as plain
-scalars.
-
-=item * BIFCODE_BIFCODE types are fully inflated into 
-Perl structures, and not the intermediate I<BIFCODE> byte string.
-
-=back
+I<bifcode> types are mapped back to Perl in the reverse way to the
+C<encode_bifcode> function, with the exception that any scalars which
+were "forced" to a particular type (using blessed references) will
+decode as unblessed scalars.
 
 Croaks on malformed data.
 
 =head2 C<force_bifcode( $scalar, $type )>
 
-Returns a reference to $scalar blessed as Bifcode::$TYPE. The value of
-$type is not checked, but the C<encode_bifcode> function will only
+Returns a reference to $scalar blessed as Bifcode::V2::$TYPE. The value
+of $type is not checked, but the C<encode_bifcode> function will only
 accept the resulting reference where $type is one of 'bytes', 'real',
 'integer' or 'utf8'.
 
 =head2 C<diff_bifcode( $bc1, $bc2, [$diff_args] )>
 
 Returns a string representing the difference between two bifcodes. The
-inputs do not need to be valid Bifcode; they are only expanded with a
-very simple regex before the diff is done. The third argument
+inputs do not need to be valid Bifcode::V2; they are only expanded with
+a very simple regex before the diff is done. The third argument
 (C<$diff_args>) is passed directly to L<Text::Diff>.
 
 Croaks if L<Text::Diff> is not installed.
 
-=head2 AnyEvent::Handle Support
-
-B<Bifcode> implements the L<AnyEvent::Handle> C<anyevent_read_type> and
-C<anyevent_write_type> functions which allow you to do this:
-
-    $handle->push_write( 'Bifcode::V2' => { your => 'structure here' } );
-
-    $handle->push_read(
-        'Bifcode::V2' => sub {
-            my ( $hdl, $ref ) = @_;
-            # do stuff with $ref
-        },
-        $maxdepth   # passed straight to decode_bifcode()
-    );
-
 =head1 DIAGNOSTICS
 
-The following exceptions may be raised by B<Bifcode>:
+The following exceptions may be raised by B<Bifcode::V2>:
 
 =over
 
-=item Bifcode::Error::Decode
+=item Bifcode::V2::Error::Decode
 
 Your data is malformed in a non-identifiable way.
 
-=item Bifcode::Error::DecodeBytes
+=item Bifcode::V2::Error::DecodeBytes
 
 Your data contains a byte string with an invalid length.
 
-=item Bifcode::Error::DecodeBytesTrunc
+=item Bifcode::V2::Error::DecodeBytesTrunc
 
 Your data includes a byte string declared to be longer than the
 available data.
 
-=item Bifcode::Error::DecodeBytesTerm
+=item Bifcode::V2::Error::DecodeBytesTerm
 
 Your data includes a byte string that is missing a "," terminator.
 
-=item Bifcode::Error::DecodeDepth
+=item Bifcode::V2::Error::DecodeDepth
 
 Your data contains dicts or lists that are nested deeper than the
 $max_depth passed to C<decode_bifcode()>.
 
-=item Bifcode::Error::DecodeTrunc
+=item Bifcode::V2::Error::DecodeTrunc
 
 Your data is truncated.
 
-=item Bifcode::Error::DecodeReal
+=item Bifcode::V2::Error::DecodeReal
 
 Your data contained something that was supposed to be a real but didn't
 make sense.
 
-=item Bifcode::Error::DecodeRealTrunc
+=item Bifcode::V2::Error::DecodeRealTrunc
 
 Your data contains a real that is truncated.
 
-=item Bifcode::Error::DecodeInteger
+=item Bifcode::V2::Error::DecodeInteger
 
 Your data contained something that was supposed to be an integer but
 didn't make sense.
 
-=item Bifcode::Error::DecodeIntegerTrunc
+=item Bifcode::V2::Error::DecodeIntegerTrunc
 
 Your data contains an integer that is truncated.
 
-=item Bifcode::Error::DecodeKeyType
+=item Bifcode::V2::Error::DecodeKeyType
 
 Your data violates the I<bifcode> format constaint that all dict keys
 be BIFCODE_BYTES or BIFCODE_UTF8.
 
-=item Bifcode::Error::DecodeKeyDuplicate
+=item Bifcode::V2::Error::DecodeKeyDuplicate
 
 Your data violates the I<bifcode> format constaint that all dict keys
 must be unique.
 
-=item Bifcode::Error::DecodeKeyOrder
+=item Bifcode::V2::Error::DecodeKeyOrder
 
 Your data violates the I<bifcode> format constaint that dict keys must
 appear in lexical sort order.
 
-=item Bifcode::Error::DecodeKeyValue
+=item Bifcode::V2::Error::DecodeKeyValue
 
 Your data contains a dictionary with an odd number of elements.
 
-=item Bifcode::Error::DecodeTrailing
+=item Bifcode::V2::Error::DecodeTrailing
 
 Your data does not end after the first I<bifcode>-serialised item.
 
-=item Bifcode::Error::DecodeUTF8
+=item Bifcode::V2::Error::DecodeUTF8
 
 Your data contained a UTF8 string with an invalid length.
 
-=item Bifcode::Error::DecodeUTF8Trunc
+=item Bifcode::V2::Error::DecodeUTF8Trunc
 
 Your data includes a string declared to be longer than the available
 data.
 
-=item Bifcode::Error::DecodeUTF8Term
+=item Bifcode::V2::Error::DecodeUTF8Term
 
 Your data includes a UTF8 string that is missing a "," terminator.
 
-=item Bifcode::Error::DecodeUsage
+=item Bifcode::V2::Error::DecodeUsage
 
 You called C<decode_bifcode()> with invalid arguments.
 
-=item Bifcode::Error::DiffUsage
+=item Bifcode::V2::Error::DiffUsage
 
 You called C<diff_bifcode()> with invalid arguments.
 
-=item Bifcode::Error::EncodeBytesUndef
+=item Bifcode::V2::Error::EncodeBytesUndef
 
 You attempted to encode C<undef> as a byte string.
 
-=item Bifcode::Error::EncodeReal
+=item Bifcode::V2::Error::EncodeReal
 
 You attempted to encode something as a real that isn't recognised as
 one.
 
-=item Bifcode::Error::EncodeRealUndef
+=item Bifcode::V2::Error::EncodeRealUndef
 
 You attempted to encode C<undef> as a real.
 
-=item Bifcode::Error::EncodeInteger
+=item Bifcode::V2::Error::EncodeInteger
 
 You attempted to encode something as an integer that isn't recognised
 as one.
 
-=item Bifcode::Error::EncodeIntegerUndef
+=item Bifcode::V2::Error::EncodeIntegerUndef
 
 You attempted to encode C<undef> as an integer.
 
-=item Bifcode::Error::EncodeUTF8Undef
+=item Bifcode::V2::Error::EncodeUTF8Undef
 
 You attempted to encode C<undef> as a UTF8 string.
 
-=item Bifcode::Error::EncodeUnhandled
+=item Bifcode::V2::Error::EncodeUnhandled
 
 You are trying to serialise a data structure that contains a data type
 not supported by the I<bifcode> format.
 
-=item Bifcode::Error::EncodeUsage
+=item Bifcode::V2::Error::EncodeUsage
 
 You called C<encode_bifcode()> with invalid arguments.
 
-=item Bifcode::Error::ForceUsage
+=item Bifcode::V2::Error::ForceUsage
 
 You called C<force_bifcode()> with invalid arguments.
 
@@ -858,13 +699,14 @@ Strings and numbers are practically indistinguishable in Perl, so
 C<encode_bifcode()> has to resort to a heuristic to decide how to
 serialise a scalar. This cannot be fixed.
 
+At the moment all Perl hash keys are encoded as BIFCODE_UTF8 as I have
+not yet had the need for BIFCODE_BYTES keys or found a cheap, obvious
+way to distinguish the two.
+
 =head1 SEE ALSO
 
 This distribution includes the L<diff-bifcode> command-line utility for
-comparing I<BIFCODE> in files.
-
-L<Bifcode::V1> implements the original (experimental) I<BIFCODE>
-version 1.
+comparing Bifcodes in files.
 
 =head1 AUTHOR
 
@@ -879,7 +721,7 @@ This software is copyright (c):
 
 =item * 2015 by Aristotle Pagaltzis
 
-=item * 2017-2019 by Mark Lawrence.
+=item * 2017-2018 by Mark Lawrence.
 
 =back
 
